Default cache key builder for Lotus.
Preserves the original key generation logic that was previously inline
in Lotus.Schema and Lotus.Cache.Key.
Default cache key builder for Lotus.
Preserves the original key generation logic that was previously inline
in Lotus.Schema and Lotus.Cache.Key.