Protocol for normalizing raw database values into JSON-safe, displayable forms.
Lotus applies this protocol to all query result values before they reach consumers (UI, exports, API). Implement this protocol for custom database types that need special handling.
Summary
Functions
Normalize a value so it is JSON-safe and human-readable.
Types
@type t() :: term()
All the types that implement this protocol.